package com.nto.data.repository

import android.content.Context
import android.content.Context.MODE_PRIVATE
import com.nto.data.models.LoginResult
import com.nto.data.models.RequestResult
import com.nto.data.models.UserDTO
import com.nto.data.models.cards.VisitCardWrapper
import com.nto.data.utils.Provider
import dagger.hilt.android.qualifiers.ApplicationContext
import okhttp3.Credentials
import javax.inject.Inject

class DataRepositoryImpl @Inject constructor(@ApplicationContext private val context: Context) :
    DataRepository {
    override suspend fun auth(login: String, password: String): LoginResult {
        val token = Credentials.basic(login, password)
        val result = Provider.provideRetrofit().auth(
            token
        ).execute()
        if (result.isSuccessful) saveToken(token, login)
        return LoginResult(result.isSuccessful, result.message())
    }

    override suspend fun saveToken(token: String, login: String) {
        context.getSharedPreferences("auth", MODE_PRIVATE).edit().putString("token", token)
            .putString("login", login).apply()
    }

    override suspend fun getToken(): String {
        return context.getSharedPreferences("auth", MODE_PRIVATE).getString("token", "")!!
    }

    override suspend fun getLogin(): String {
        return context.getSharedPreferences("auth", MODE_PRIVATE).getString("login", "")!!
    }

    override suspend fun getInfo(login: String): UserDTO {
        val result = Provider.provideRetrofit().getInfo(token = getToken(), login = login).execute()
        return if (result.isSuccessful) result.body()!!
        else UserDTO(isError = true, isUnauthorized = result.code() == 403)
    }

    override suspend fun getVisits(id: String): VisitCardWrapper {
        val result = Provider.provideRetrofit().getVisits(getToken(), id.ifBlank { getLogin() }).execute()
        return if (result.isSuccessful) {
            VisitCardWrapper(result.body()!!)
        } else {
            VisitCardWrapper(null, true)
        }
    }

    override suspend fun open(): RequestResult {
        val result = Provider.provideRetrofit().open(getToken()).execute()
        return when (result.code()) {
            200 -> RequestResult.OK
            400, 403 -> RequestResult.CANCELED
            else -> RequestResult.ERROR
        }
    }

    override suspend fun logout() {
        context.getSharedPreferences("auth", MODE_PRIVATE).edit().remove("token").remove("login")
            .apply()
    }

    override suspend fun invertLock(login: String, status: Boolean): RequestResult {
        return when (Provider.provideRetrofit().invertLock(getToken(), login, status).execute()
            .code()) {
            200 -> RequestResult.OK
            400, 403 -> RequestResult.CANCELED
            else -> RequestResult.ERROR
        }
    }

}