package com.nto.presentation.composable

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.Stable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Shape
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.nto.presentation.composable.DecoratedButtonType.Default
import com.nto.presentation.composable.DecoratedButtonType.Disabled
import com.nto.presentation.theme.NTOTheme
import com.nto.presentation.theme.raleway
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow

enum class DecoratedButtonType {
    Default, Disabled
}

object DecoratedButtonValues {
    private val DefaultColors: ButtonColors
        @Composable get() = ButtonColors(
            contentColor = NTOTheme.colors.primaryBackground,
            containerColor = NTOTheme.colors.button,
            disabledContentColor = Color.Unspecified,
            disabledContainerColor = Color.Unspecified
        )

    private val DisabledColors: ButtonColors
        @Composable get() = ButtonColors(
            disabledContentColor = NTOTheme.colors.primaryBackground,
            disabledContainerColor = NTOTheme.colors.buttonDisabled,
            contentColor = NTOTheme.colors.primaryBackground,
            containerColor = NTOTheme.colors.button
        )

    @Composable
    fun getDefaultColor(type: DecoratedButtonType): ButtonColors {
        return when (type) {
            Default -> DefaultColors
            Disabled -> DisabledColors
        }
    }
}

/**
 * High level element that utilizes [DecoratedButtonType] to obtain [DecoratedButtonValues] and use it's colors accordingly.
 *
 * Consists of a button with a text inside it. By fact that's just a wrapper to simplify reusing process.
 *
 * @param text text to display inside a button.
 * @param isDisabled mutable variable that represents button state. Should be in viewmodel or screen state. After being converted to the instance of [DecoratedButtonType] that is used to obtain [ButtonColors] inside [DecoratedButtonValues].
 * @param modifier modifier that should contain [Modifier.height] and [Modifier.width] or other size definition to work correctly.
 * @param shape element will be clipped to that shape.
 * @param onClick function to be invoked on button click.
 *
 * @sample [DecoratedButtonSample]
 *
 */

@Composable
fun DecoratedButton(
    text: String,
    isDisabled: Boolean,
    modifier: Modifier = Modifier,
    shape: Shape = RoundedCornerShape(10.dp),
    onClick: () -> Unit,
) {
    NTOTheme {
        Button(
            modifier = modifier, shape = shape, colors = DecoratedButtonValues.getDefaultColor(
                when (isDisabled) {
                    true -> Disabled
                    false -> Default
                }
            ), onClick = onClick,
            enabled = !isDisabled
        ) {
            Text(
                text,
                fontWeight = FontWeight.SemiBold,
                fontSize = 14.sp,
                fontFamily = raleway,
                color = NTOTheme.colors.secondaryText
            )
        }
    }
}

@Stable
internal class DecoratedButtonSample {
    class SampleViewModel {
        private val _state = MutableStateFlow(SampleState())

        val state: StateFlow<SampleState>
            get() = _state.asStateFlow()

        fun setDisabledState(data: Boolean) {
            _state.value = _state.value.copy(data = data)
        }
    }

    data class SampleState(
        var data: Boolean = false
    )

    @Preview
    @Composable
    private fun InputFieldPreview() {
        NTOTheme {
            val sampleViewModel = SampleViewModel()
            val state = sampleViewModel.state.collectAsState()
            Column(modifier = Modifier.fillMaxSize()) {
                //Filled button preset
                DecoratedButton(
                    text = "Get Started",
                    isDisabled = state.value.data,
                    modifier = Modifier
                        .height(60.dp)
                        .width(300.dp),
                    shape = RoundedCornerShape(10.dp)
                ) {
                    //...
                }
                //Disabled button preset
                Spacer(modifier = Modifier.height(10.dp))
                DecoratedButton(
                    text = "Get Started",
                    isDisabled = !state.value.data,
                    modifier = Modifier
                        .height(60.dp)
                        .width(300.dp),
                    shape = RoundedCornerShape(10.dp)
                ) {
                    //...
                }
            }
        }
    }
}