package com.nto.presentation.screens.profileScreen

import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.IconButtonDefaults
import androidx.compose.material3.Text
import androidx.compose.material3.pulltorefresh.PullToRefreshBox
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import androidx.navigation.compose.rememberNavController
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanOptions
import com.nto.data.utils.Destinations
import com.nto.presentation.CustomCaptureActivity
import com.nto.presentation.R
import com.nto.presentation.composable.DecoratedButton
import com.nto.presentation.composable.cards.VisitCard
import com.nto.presentation.theme.NTOTheme
import com.nto.presentation.theme.TextGray
import com.nto.presentation.theme.playfair
import com.nto.presentation.theme.raleway

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ProfileScreen(
    navController: NavController,
    modifier: Modifier = Modifier,
    viewModel: ProfileViewModel = hiltViewModel<ProfileViewModel>(),
) {
    val state = viewModel.state.collectAsState().value
    val scannerLauncher = rememberLauncherForActivityResult(
        contract = ScanContract(),
        onResult = { result -> navController.navigate(Destinations.Scan(result.contents)) }
    )
    val scanOptions = ScanOptions()
    scanOptions.setPrompt("")
    scanOptions.setBeepEnabled(false)
    scanOptions.setDesiredBarcodeFormats(ScanOptions.QR_CODE)
    scanOptions.setOrientationLocked(false)
    scanOptions.setCaptureActivity(CustomCaptureActivity::class.java)

    LaunchedEffect(state.isUnauthorized) {
        if (state.isUnauthorized) {
            navController.navigate(Destinations.Login)
        }
    }
    PullToRefreshBox(
        isRefreshing = viewModel.isLoading,
        onRefresh = {
            viewModel.updateInfo()
        }
    ) {
        Column(
            modifier = modifier
                .background(NTOTheme.colors.primaryBackground)
        ) {
            Column(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(start = 20.dp, end = 20.dp)
                    .verticalScroll(
                        rememberScrollState()
                    )
            ) {
                Spacer(modifier = Modifier.height(10.dp))
                Column(modifier = Modifier.weight(1f)) {
                    Row(
                        modifier = Modifier.fillMaxWidth(),
                        horizontalArrangement = Arrangement.SpaceBetween,
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(
                            stringResource(R.string.title_profile),
                            fontFamily = playfair,
                            color = NTOTheme.colors.primaryText,
                            fontSize = 32.sp,
                            fontWeight = FontWeight.Bold
                        )
                        Row(verticalAlignment = Alignment.CenterVertically) {
                            IconButton(
                                onClick = {
                                    navController.navigate(Destinations.Admin)
                                },
                                modifier = Modifier
                                    .size(44.dp)
                                    .border(
                                        BorderStroke(2.dp, NTOTheme.colors.buttonAdmin),
                                        CircleShape
                                    ),
                            ) {
                                Icon(
                                    painter = painterResource(R.drawable.icon_admin),
                                    contentDescription = null,
                                    tint = NTOTheme.colors.buttonAdmin
                                )
                            }
                            Spacer(modifier = Modifier.width(12.dp))
                            IconButton(
                                onClick = {},
                                modifier = Modifier
                                    .size(44.dp)
                                    .border(
                                        BorderStroke(2.dp, NTOTheme.colors.buttonDisabled),
                                        CircleShape
                                    )
                            ) {
                                Icon(
                                    painter = painterResource(R.drawable.icon_options),
                                    contentDescription = null,
                                    tint = NTOTheme.colors.buttonDisabled
                                )
                            }
                            Spacer(modifier = Modifier.width(16.dp))
                            IconButton(
                                modifier = Modifier.size(44.dp), onClick = {
                                    viewModel.logout(navController)
                                }, colors = IconButtonDefaults.iconButtonColors(
                                    containerColor = NTOTheme.colors.buttonDisabled
                                )
                            ) {
                                Icon(
                                    painter = painterResource(R.drawable.icon_logout),
                                    contentDescription = null,
                                    tint = NTOTheme.colors.primaryBackground
                                )
                            }
                        }

                    }
                    Spacer(modifier = Modifier.height(50.dp))
                    Row(
                        modifier = Modifier.fillMaxWidth(),
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Image(
                            painter = painterResource(R.drawable.logo_placeholder_user),
                            modifier = Modifier
                                .size(100.dp)
                                .clip(
                                    CircleShape
                                ),
                            contentDescription = null
                        )
                        Spacer(modifier = Modifier.width(15.dp))
                        Column(
                            modifier = Modifier
                                .fillMaxWidth()
                                .clip(RoundedCornerShape(10.dp))
                                .background(NTOTheme.colors.inputFieldBackground)
                                .padding(10.dp)
                        ) {
                            Text(
                                state.secondName,
                                fontFamily = raleway,
                                color = NTOTheme.colors.primaryText,
                                fontSize = 16.sp,
                                fontWeight = FontWeight.SemiBold,
                                lineHeight = 16.sp
                            )
                            Text(
                                state.firstName,
                                fontFamily = raleway,
                                color = NTOTheme.colors.primaryText,
                                fontSize = 16.sp,
                                fontWeight = FontWeight.SemiBold,
                                lineHeight = 16.sp
                            )
                            Text(
                                state.thirdName,
                                fontFamily = raleway,
                                color = NTOTheme.colors.primaryText,
                                fontSize = 16.sp,
                                fontWeight = FontWeight.SemiBold,
                                lineHeight = 16.sp
                            )
                            Spacer(modifier = Modifier.height(8.dp))
                            Text(
                                state.job,
                                fontWeight = FontWeight.Medium,
                                fontFamily = raleway,
                                fontSize = 14.sp,
                                color = TextGray
                            )
                        }
                    }
                    Spacer(modifier = Modifier.height(20.dp))
                    Row(
                        modifier = Modifier
                            .fillMaxWidth()
                            .height(35.dp)
                            .clip(RoundedCornerShape(10.dp))
                            .background(NTOTheme.colors.inputFieldBackground)
                            .padding(start = 15.dp, end = 15.dp),
                        verticalAlignment = Alignment.CenterVertically,
                        horizontalArrangement = Arrangement.SpaceBetween
                    ) {
                        Text(
                            stringResource(R.string.label_last_visit),
                            fontWeight = FontWeight.Medium,
                            fontFamily = raleway,
                            color = NTOTheme.colors.primaryText,
                            fontSize = 14.sp
                        )
                        Text(
                            state.lastOpen,
                            fontWeight = FontWeight.Normal,
                            fontFamily = raleway,
                            color = NTOTheme.colors.primaryText,
                            fontSize = 14.sp
                        )
                    }
                    Spacer(modifier = Modifier.height(20.dp))
                    Text(
                        stringResource(R.string.label_visits),
                        fontWeight = FontWeight.SemiBold,
                        fontSize = 20.sp,
                        fontFamily = raleway,
                        color = NTOTheme.colors.primaryText
                    )
                    Spacer(modifier = Modifier.height(10.dp))

                    LazyColumn(verticalArrangement = Arrangement.spacedBy(10.dp)) {
                        items(state.visits) { item ->
                            VisitCard(item)
                        }
                    }
                }
                DecoratedButton(
                    stringResource(R.string.label_scan),
                    false,
                    modifier = Modifier
                        .padding(bottom = 10.dp)
                        .fillMaxWidth()
                        .height(62.dp)
                ) {
                    scannerLauncher.launch(scanOptions)
                }
            }
        }
    }

}

@Preview
@Composable
private fun ProfileScreenPreview() {
    NTOTheme {
        ProfileScreen(rememberNavController(), Modifier.fillMaxSize())
    }
}